#!/usr/bin/perl
#
# Thumbnail and html page generator
# Copyright Arto Ters <ajt@iki.fi> 1999-2003
#
# Some inspiration from thumbs.sh by Adam Kopacz / kLoGraFX
# http://www.klografx.de/ - Adam.K@klografx.de
#
# Version: 1.02
# Licence: GNU General Public License (version 2 or any later version)
#          http://www.gnu.org/copyleft/gpl.txt
#
# Changelog:
#
# 1.0  Initial version
# 1.01 Fixed a small error in command line utilities detection
# 1.02 Fixed parsing problem with recent versions of ImageMagick
#      and added rounding image sizes to nearest integer if 
#      dividing with SCALEFACTOR produced a non-integer. 
#
# Latest version: http://www.iki.fi/ajt/software/thumbnailtools/
######################################################################
#
# Global defaults:
# 
$SCALEFACTOR = 4;    # the size of the original picture is divided
                     # by the SCALEFACTOR to get the thumbnail size,
                     # can be adjusted using the command line option -d

$THUMBQUALITY = 75;  # thumbnail image quality, can be adjusted using 
                     # the command line option -q

$PAGE_TITLE = "Some pictures";  # The default title, can be adjusted
                                # using the command line option -s

$OUTPUT_FILE = "index.html";    # The default output file, can be adjusted
                                # using the command line option -o

$ASK_OVERWRITE = 1;  # If one, asks the user whether the output file
                     # should be overwritten in case it exits.
                     # (not adjustable from command line)

$TIMESTAMP_FROM_ORIGINAL = 1;   # file date picked from the 
                                # picture_orig.jpg, if it exists
                                # (not adjustable from command line)

$DO_THUMBNAILS = 1;  # if zero, just regenerates the html page (fast),
                     # can be set to zero using the command line 
                     # flag -n

#
######################################################################

use Getopt::Std;
use POSIX;

# Modify the html code to suit your needs. 
sub print_html_header
{
    my $fh = shift;
    my $title = shift;
    my $h1 = shift;
    
    print $fh "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">\n";
    print $fh "<html>\n";
    print $fh "<head>\n";
    print $fh "<title>$title</title>\n";
    print $fh "</head>\n";
    
    print $fh "<body BGCOLOR=\"\#ffffff\">\n";
    print $fh "<h1 align=\"center\">$h1</h1>\n";
    print $fh "<table border=\"0\">\n\n";
}

# Modify the html code to suit your needs. 
sub print_html_footer
{
    my $fh = shift;
    my @time = localtime(time);
    my $datestr = POSIX::strftime("%d.%m.%Y.", @time);
    my $yearstr = POSIX::strftime("%Y", @time);
    print $fh "<tr>\n";
    print $fh "<td colspan=\"2\">\n";
    print $fh "<hr>\n";
    print $fh "<address>\n";
    print $fh "Copyright Unknown Author <a href=\"mailto:unknown\@some.domain\">";
    print $fh "&lt\;unknown\@some.domain&gt\;</A> ${yearstr}.<br>\n";
    print $fh "Redistribution of this document as a whole or any of the pictures\n";
    print $fh "individually is permitted in any medium provided this copyright\n";
    print $fh "notice is preserved.\n";
    
    my $time = scalar localtime;
    print $fh "<p>Last update $datestr\n";
    print $fh "</address>\n";
    print $fh "</td>\n";
    print $fh "</tr>\n";
    
    print $fh "</table>\n";
        
    print $fh "</body>\n";
    print $fh "</html>\n";
}

sub find_program {
    # Quick check if the given program can be found in the PATH
    # This will work only on Unix machines...
    my $program = shift;
    @system_dirs = split(':', $ENV{PATH});

    foreach (@system_dirs) {
	if (-x "$_/${program}") {
	    return 1;
	}
    }    
    # not found
    return 0;
}

# The main program
#
getopts('d:no:q:s:');

if (@ARGV) 
{
    if ($opt_d != 0) {
	$SCALEFACTOR = $opt_d;
    }
    if ($opt_n == 1) {
	$DO_THUMBNAILS = 0;
    }
    if ($opt_o ne "") {
	$OUTPUT_FILE = $opt_o;
    }
    if ($opt_q != 0) {
	$THUMBQUALITY = $opt_q;
    }
    if ($opt_s ne "") {
	$PAGE_TITLE = $opt_s;
    }

    # Check for necessary command line utilities
    $USE_IMAGEMAGICK = 0;
    if (! (find_program("cjpeg") == 1 
	   && find_program("djpeg") == 1
	   && find_program("pnmscale") == 1
	   && find_program("identify") == 1)) {
	# Ok, not all the programs available, but we can do the same a bit
	# slower with the convert utility from ImageMagick
	$USE_IMAGEMAGICK = 1;
	if (find_program("convert") == 0
	    || find_program("identify") == 0) {
	    print STDERR "Couldn't find all the necessary utilities.\n\n";
	    print STDERR "Please make sure that following programs can be found in the PATH:\n"; 
	    print STDERR "1. identify (ImageMagick package)\n";
	    print STDERR "2. convert (ImageMagick package)\n";
	    print STDERR "(works faster if you also have cjpeg, djpeg and pnmscale)\n";
	    exit(1);
	}
    }

    # Open the output file
    if (-f $OUTPUT_FILE) {
	if ($ASK_OVERWRITE) {
	    print "$OUTPUT_FILE exists, overwrite (y/n)? ";
	    my $response = getc;
	    if ($response ne "y" && $response ne "Y") {
		die "Quitting...";
	    }
	}
    }
	    
    open(OUTFILE, ">$OUTPUT_FILE")
	or die "Couldn't open $OUTPUT_FILE for output\n";

    # The title and H1 header of the page are the same
    print_html_header(\*OUTFILE, $PAGE_TITLE, $PAGE_TITLE);

    my $image;
    foreach $image (@ARGV) 
    {		
	if ($image =~ /(.*)\.jpg/)
	{	    
	    print "Processing image $image\n";
	    my $basename = $1;
	    my $thumbname = "$basename" . "_small.jpg";
	    my $origname = "$basename" . "_orig.jpg";

	    my @imagestats = stat($image);
	    my $imagesize = int ($imagestats[7] / 1024);  # in kilobytes
	    if (($TIMESTAMP_FROM_ORIGINAL) && (-f $origname))
	    {
		@imagestats = stat($origname);
	    }
	    my $imagedate = scalar localtime($imagestats[9]);
	    
	    # If the image name contains spaces, parsing the output of identify
	    # would fail without this trick:
	    my $parts_in_image_name = scalar(split(' ', $image));
	    my @imageinfo = split(' ', `identify -ping "$image"`);
	    my $dimensions = $imageinfo[$parts_in_image_name+1];
	    # In older versions of ImageMagick identify gave the image dimensions
	    # as first element after image name, so let's check that we got it 
	    # correctly and try the older format too.
	    if (! ($dimensions =~ /\d+x\d+/)) {
		$dimensions = $imageinfo[$parts_in_image_name];
		if (! ($dimensions =~ /\d+x\d+/)) {
		    die "Couldn't figure out the image dimensions from identify program output.\n";
		}
	    }
	    my ($xsize, $ysize) = split('x', $dimensions);
	    $xsize /= $SCALEFACTOR;
	    $ysize /= $SCALEFACTOR;
	    $xsize = POSIX::floor($xsize+0.5); # Make sure the size is a valid integer
	    $ysize = POSIX::floor($ysize+0.5);
	    
	    # Generate thumbnail
	    if ($DO_THUMBNAILS) {
		if ($USE_IMAGEMAGICK) {
		    `convert -geometry ${xsize}x${ysize} -quality ${THUMBQUALITY} -interlace Line "$image" "$thumbname"`;
		}
		else {
		    # This is noticeably faster 
		    `djpeg -pnm "$image" | pnmscale -xsize $xsize | cjpeg -quality $THUMBQUALITY -optimize -progressive > "$thumbname"`;
		}
	    }

	    # Modify the html code to suit your needs. 
	    print OUTFILE "<tr>\n";
	    print OUTFILE "<td><a href=\"$image\">\n";
	    print OUTFILE "<img src=\"$thumbname\" width=\"$xsize\" height=\"$ysize\" alt=\"$image\"></a>\n";
	    print OUTFILE "</td>\n";
	    print OUTFILE "<td>\n";
	    print OUTFILE "$imagedate<br>\n";
	    print OUTFILE "($imagesize kB)\n";
	    print OUTFILE "</td>\n";
	    print OUTFILE "</tr>\n\n";
	}
	else
	{
	    print "Skipping image $image\n";
	}
    }

    print_html_footer(\*OUTFILE);
    close(OUTFILE);
    exit(0);
}
else
{
    print "Usage: thumbgenerator.pl [OPTIONS] IMAGEFILES (in jpeg format)\n\n";
    print "Available options:\n";
    print "-d divider     the scaling factor for thumbnail size (default ${SCALEFACTOR})\n";
    print "-n             if set, just regenerate html (no thumbnail generation)\n";
    print "-o name        the output file name (default ${OUTPUT_FILE})\n"; 
    print "-q quality     the output jpeg quality (default ${THUMBQUALITY}%)\n"; 
    print "-s subject     the page title (default \"${PAGE_TITLE}\")\n"; 
    exit(1);
}

